/**
 * Add a map dialog.
 *
 * The `.externalcontentUiBingmapdialog()` method adds functionality to open a map for a specified location.
 *
 *
 * ### Examples
 * Apply .externalcontentUiBingmapdialog() to element.
 *
 * JavaScript:
 *
 *      require([
 *        'de_epages/externalcontent/ui/bingmapdialog',
 *
 *        '$ready!'
 *      ], function(de_epages) {
 *
 *          de_epages('#ShippingMap').externalcontentUiBingmapdialog({
 *            streetID:'#Street[js,html]',
 *            zipID:'#Zipcode[js,html]',
 *            cityID:'#City[js,html]',
 *            countryID:'#Country.English[js,html]'
 *          })
 *      });
 *
 * HTML:
 *
 *    <span id="ShippingMap" class="AlignMiddle NoWrap">
 *      <i class="fa fa-globe fa-2x ClickIcon"></i>
 *      <a href="#" onclick="return false;">Auf Karte anzeigen</a>
 *    </span>
 *
 *
 * @class jQuery.ui.externalcontentUiBingmapdialog
 *
 * @uses de_epages
 * @uses ep
 * @uses ep.ui.dialog
 * @uses jquery.ui.widget
 * @since 7.9.0
 */

/**
 * @cfg {streetID}      String or function returning string     Streetname
 * @cfg {zipID}         String or function returning string     zipcode
 * @cfg {cityID}        String or function returning string     cityname
 * @cfg {countryID}     String or function returning string     countryname
 */

/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/externalcontent/ui/bingmapdialog
 */
define('de_epages/externalcontent/ui/bingmapdialog', [
    'jquery',
    'ep',
    'de_epages',

    'jquery/ui/widget',
    'ep/ui/dialog'
], function ($, ep, de_epages) {

    $.widget('ui.externalcontentUiBingmapdialog', {

        options: {
            streetID: "Street",
            zipID: "Zipcode",
            cityID: "City",
            countryID: "CountryID"
        },

        _create: function() {
            var self = this,
                elem = self.element;

            // register open dialog action to batch action button
            elem.on('click', function(evt) {
                self._openDialog();
            });
        },

        _escape: function(string) {
            // escape url-forbidden characters
            return string.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
        },

        _encode: function(string) {
            // summary:
            //     prepare a string to use it with HTTP/Get
            // description:
            //     Transfer chars which can't be used in a GET string to ones which can
            //       e.q. ü,ä,ö,...
            // example:
            //     | var result = _encode("öäü");
            // returns:
            //     encoded string

            string = string.replace(/\r\n/g,"\n");
            var utftext = "";

            for (var n = 0,l=string.length; n < l; n++) {

                var c = string.charCodeAt(n);

                if (c < 128) {
                    utftext += String.fromCharCode(c);
                }
                else if((c > 127) && (c < 2048)) {
                    utftext += String.fromCharCode((c >> 6) | 192);
                    utftext += String.fromCharCode((c & 63) | 128);
                }
                else {
                    utftext += String.fromCharCode((c >> 12) | 224);
                    utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                    utftext += String.fromCharCode((c & 63) | 128);
                }
            }
            return utftext;
        },

        _generateSearchString: function() {
            var self = this,
                o = self.options,
                searchString = "";

            // insert create string from options (properties can be functions or strings)
            searchString = "s=" + ((typeof o.streetID === 'string') ? o.streetID : o.streetID())  + "&" + "z=" + ((typeof o.zipID === 'string') ? o.zipID : o.zipID())  + "&" + "c=" + ((typeof o.cityID === 'string') ? o.cityID : o.cityID())  + "&" + "i=" + ((typeof o.countryID === 'string') ? o.countryID : o.countryID()) ;

            return escape(this._encode(searchString));
        },


        _openDialog: function() {
            var self = this,
                o = self.options,
                searchString = self._generateSearchString(),
                dialogContent;

            // create iframe

            // open if dialog exists, otherwise create new dialog
            if (self.dialog) {

                self.dialog.uiDialog('open');
            } else {
                dialogContent = $('<div style="padding-top:0px;"><iframe src="?ViewAction=UnityMBO-ViewBingMap&' + searchString + '" style="height:500px; width: 470px">Alternativtext</iframe></div>');

                self.dialog = dialogContent.uiDialog({
                    modal: true,
                    autoOpen: true,
                    width: 500,
                    resizeable: false
                });
            }
        },

        destroy: function () {
            var self = this;

            self.dialog.uiDialog('destroy');
            self._superApply(arguments);
        }
    });

    return de_epages;
});
